from ingestion.vcf_standardization.util.af_helpers import try_move_af_value, try_calculate_af
from ingestion.vcf_standardization.util.dp_helpers import (
    try_move_dp_value,
    try_afdp_to_dp,
    try_calculate_dp_from_ad,
)


class Variant:
    def __init__(self, fields):
        self.chr = fields[0]
        self.pos = fields[1]
        self.rsid = fields[2]
        self.ref = fields[3]
        self.alt = fields[4]
        self.qual = fields[5]
        self.filt = fields[6]
        self.info = {x.split("=")[0]: x.split("=")[1] for x in fields[7].split(";") if "=" in x}
        self.frmt = fields[8].split(":")
        self.smpl = fields[9].split(":")
        self.ad_af_dp = {"AD": False, "AF": False, "DP": False}

    def standardize_allele_frequency(self, log):
        # Detect if allele frequency is present either in the INFO or FORMAT/SAMPLE fields
        for k, v in self.info.items():
            if k in ["af", "AF"]:
                # Cast to float, update dict, and remove old entry
                new_entry = "{:.4f}".format(float(v))
                self.info.pop(k)
                self.info.update({"AF": new_entry})

                # Replace with standardized AF
                self.ad_af_dp["AF"] = new_entry
                break

        # If AF is not present in INFO, check FORMAT/SAMPLE fields
        if not self.ad_af_dp["AF"]:
            self = try_move_af_value(self)

            # Last resort, calculate AF from AD
            if not self.ad_af_dp["AF"] and self.ad_af_dp["AD"]:
                self = try_calculate_af(self)

        return self

    def standardize_depth(self, log):
        # Check for depth in FORMAT, if there, all is good
        for entry in zip(self.frmt, self.smpl):
            if entry[0] == "DP":
                self.ad_af_dp["DP"] = entry[1]
                break

        if not self.ad_af_dp["DP"]:
            # search for DP or depth in INFO, move to FORMAT/SAMPLE
            self = try_move_dp_value(self)

            # search for AFDP in FORMAT
            if not self.ad_af_dp["DP"]:
                self = try_afdp_to_dp(self)

                # Last resort, calculate DP from AD
                if not self.ad_af_dp["DP"] and self.ad_af_dp["AD"]:
                    self = try_calculate_dp_from_ad(self)

        return self

    def standardize_allelic_depth(self, log):
        if not self.ad_af_dp["AD"] and self.ad_af_dp["AF"] and self.ad_af_dp["DP"]:
            alt_depth = round(float(self.ad_af_dp["DP"]) * float(self.ad_af_dp["AF"]))
            ref_depth = int(self.ad_af_dp["DP"]) - alt_depth
            ad = f"{ref_depth},{alt_depth}"
            self.ad_af_dp["AD"] = ad
            self.frmt.append("AD")
            self.smpl.append(ad)

        return self

    def standardize_genotype(self, log):
        if not "GT" in self.frmt and self.ad_af_dp["AF"]:
            gt = "1/1" if float(self.ad_af_dp["AF"]) > 0.9 else "0/1"
            self.frmt.append("GT")
            self.smpl.append(gt)

        return self

    def add_vendsig(self, vendsig_dict, log):
        # Add VENDSIG to INFO
        vendor = vendsig_dict.get("vendor", "")
        if not vendor:
            return self

        if vendor == "caris":
            vendsig_lookup = vendsig_dict.get(
                f"{self.chr}:{self.pos}:{self.ref}:{self.alt}", "Unknown"
            )
            self.info.update({"VENDSIG": vendsig_lookup})

        elif vendor == "foundation":
            var_gene_id = self.info.get("gene_name")
            var_depth = self.info.get("depth")
            vendsig_lookup = vendsig_dict.get(f"{var_gene_id}:{var_depth}", "Unknown")
            self.info.update({"VENDSIG": vendsig_lookup})

        return self

    def prune_var(self, log):
        # Pruning FORMAT to only include GT, AD, and DP values.
        # Pruning VARIANT column to only include the corresponding values.
        # Pruning INFO column to only include AF and VENDSIG values.
        zipped = dict(zip(self.frmt, self.smpl))
        frmt_smpl_dict = {frmt: smpl for frmt, smpl in zipped.items() if frmt in ["GT", "AD", "DP"]}
        self.frmt = list(frmt_smpl_dict.keys())
        self.smpl = list(frmt_smpl_dict.values())
        self.info = {k: v for k, v in self.info.items() if k in ["AF", "VENDSIG"]}

    def reassemble_variant(self):
        # Reassemble variant into string
        updated_variant = "\t".join(
            [
                self.chr,
                self.pos,
                self.rsid,
                self.ref,
                self.alt,
                self.qual,
                self.filt,
                ";".join([f"{k}={v}" for k, v in self.info.items()]),
                ":".join(self.frmt),
                ":".join(self.smpl),
            ]
        )
        return updated_variant


def check_formatting(var: str) -> Variant:
    # Loose formatting check, return as Variant class object
    split_var = var.split("\t")
    if len(split_var) < 8 or not split_var[1].isdigit():
        raise RuntimeError(f"Variant contains incorrect number, or invalid fields:  {var}")

    if len(split_var) == 8:
        split_var.append(".")  # Add placeholder for FORMAT
        split_var.append(".")  # Add placeholder for SAMPLE

    elif len(split_var) == 9:
        split_var.append(".")  # Add placeholder for SAMPLE

    working_variant = Variant(split_var)
    return working_variant



def try_move_dp_value(self):

    # Info section -> get depth or dp
    depth = self.info.get("depth", False)
    if not depth:
        depth = self.info.get("DP", False)
        if not depth:
            return self

    self.frmt.append("DP")
    self.smpl.append(depth)
    self.ad_af_dp["DP"] = depth
    return self


def try_afdp_to_dp(self):
    # Rename AFDP to just DP in FORMAT for each variant
    for entry in zip(self.frmt, self.smpl):
        if entry[0] == "AFDP":
            self.frmt[self.frmt.index("AFDP")] = "DP"
            self.ad_af_dp["DP"] = entry[1]
            break
    return self


def try_calculate_dp_from_ad(self):
    depth = str(sum([int(i) for i in self.ad_af_dp["AD"].split(",")]))
    self.frmt.append("DP")
    self.smpl.append(depth)
    self.ad_af_dp["DP"] = depth
    return self



import gzip
import re
import os
from typing import Iterator, Optional


def check_vcf(infile, log):
    log.info("Checking VCF file")
    # Check if file exists. Raise if it doesn't.
    if os.path.exists(infile) == False:
        raise RuntimeError(f'Given file path "{infile}" could not be located')

    if not (infile.endswith((".vcf", ".vcf.gz"))):
        raise RuntimeError(f'Given file "{infile}" must be in vcf or vcf.gz format')


def read_variants(f) -> Iterator[str]:
    for line in f:
        record = re.sub(" ", "", line.rstrip("\r\n"))
        if record.startswith("#"):
            continue
        yield record


def read_headers(f):
    headers = []
    for line in f:
        record = line.rstrip("\r\n")
        if record.startswith("#"):
            if not record.startswith("##INFO"):
                headers.append(record)
            else:
                if record.split(",")[1] == "Number=0" and record.split(",")[2] != "Type=Flag":
                    continue
                headers.append(record)

        else:
            break

    return headers


def add_to_headers(headers: list, case_id) -> list:
    headers_to_add = [
        '##INFO=<ID=AF,Number=A,Type=Float,Description="Allele frequency, for each ALT allele, in the same order as listed">',
        '##FORMAT=<ID=AD,Number=.,Type=Integer,Description="Number of reads harboring allele (in order specified by GT)">',
        '##FORMAT=<ID=DP,Number=1,Type=Integer,Description="Read depth">',
        '##FORMAT=<ID=GT,Number=1,Type=String,Description="Genotype">',
        '##INFO=<ID=VENDSIG,Number=1,Type=String,Description="Vendor Significance">',
    ]
    for new_header in headers_to_add:
        if new_header not in headers:
            headers.insert(-1, new_header)

    case_id_header = f"#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t{case_id}"

    if case_id_header not in headers:
        headers.insert(-1, case_id_header)
        return headers[:-1]

    return headers


def write_vcf(
    headers: list,
    variants_gen: Iterator[Optional[str]],
    outfile: str,
    compression: bool,
    line_count: int,
    log,
):
    log.info(f"Writing standardized VCF to {outfile}")

    with gzip.open(outfile, "wt") if compression else open(outfile, "w") as w:
        w.write("\n".join(headers) + "\n")
        for variant in variants_gen:
            line_count += 1
            if variant:
                w.write(variant + "\n")

    return line_count



import datetime
import gzip
import io
import os
import re
import subprocess
import sys
import zipfile

from logging import Logger

from ingestion.vcf_standardization.standardize import standardize_vcf


# This is done in next step, we are just adding to yaml
def extract_sv(prefix, ingest_status):
    vcfs = []

    # Hard-code genome reference for Caris VCFs
    genome_reference = "GRCh38"

    if ingest_status["run_instructions"]["som_vcf"]:
        vcfs.append(
            {
                "fileName": f".lifeomic/caris/{prefix}/{prefix}.modified.somatic.nrm.filtered.vcf.gz",
                "sequenceType": "somatic",
                "type": "shortVariant",
                "reference": genome_reference,
            }
        )

    if ingest_status["run_instructions"]["germ_vcf"]:
        vcfs.append(
            {
                "fileName": f".lifeomic/caris/{prefix}/{prefix}.modified.germline.nrm.filtered.vcf.gz",
                "sequenceType": "germline",
                "type": "shortVariant",
                "reference": genome_reference,
            }
        )

    return vcfs


def get_vendsig_dict(json_data, log: Logger):
    # Return a dicitionary of {'chr:star_pos:ref:alt' : 'vendsig'}
    vendsig_dict = {"vendor": "caris"}
    for test in json_data["tests"]:
        results = [
            result
            for result in test.get("testResults", {})
            if isinstance(result, dict) and "genomicAlteration" in result.keys()
        ]
        for result in results:
            if "alterationDetails" in result["genomicAlteration"].keys():
                vendsig = map_vendsig(result["genomicAlteration"]["result"])
                sv = result["genomicAlteration"]["alterationDetails"]["transcriptAlterationDetails"]
                vendsig_dict.update(
                    {
                        f'{result["genomicAlteration"]["chromosome"]}:{sv["transcriptStartPosition"]}:{sv["referenceNucleotide"]}:{sv["observedNucleotide"]}': vendsig
                    }
                )

    return vendsig_dict


def map_vendsig(ci: str) -> str:
    if ci in ["Pathogenic Variant", "Pathogenic"]:
        return "Pathogenic"
    elif ci in ["Likely Pathogenic Variant", "Likely Pathogenic"]:
        return "Likely pathogenic"
    elif ci in ["Benign Variant", "Benign"]:
        return "Benign"
    elif ci in ["Likely Benign Variant", "Likely Benign"]:
        return "Likely benign"
    elif ci in ["Variant of Uncertain Significance", "VUS"]:
        return "Uncertain significance"
    else:
        return "Unknown"


def process_caris_vcf(infile, json_data, outpath, file_name, log: Logger):
    line_count = 0
    vendsig_dict = {"vendor": "caris"}

    if "germline.vcf" in infile:
        outfile = f"{file_name}.modified.germline.vcf"
        sample_name = "germline_" + file_name

    else:
        outfile = f"{file_name}.modified.somatic.vcf"
        sample_name = file_name
        # Read in a dictionary of variants with VENDSIG from the JSON file for somatic only
        vendsig_dict = get_vendsig_dict(json_data, log)

    line_count = standardize_vcf(
        infile=infile,
        outfile=outfile,
        out_path=outpath,
        case_id=sample_name,
        log=log,
        vendsig_dict=vendsig_dict,
        compression=True,
    )

    return line_count



import gzip
from logging import Logger
from typing import Iterator, Optional

from ingestion.vcf_standardization.util.read_write import (
    add_to_headers,
    write_vcf,
    check_vcf,
    read_headers,
    read_variants,
)
from ingestion.vcf_standardization.Variant import check_formatting


def format_variant(variant: str, log: Logger, vendsig_dict: dict = None) -> Optional[str]:
    # Ignore structural variants
    if "SVTYPE" in variant:
        return None
    # Working variant
    wv = check_formatting(variant)

    # Only process variants that aren't multiallelic
    if len(wv.alt.split(",")) == 1:
        if "AD" in wv.frmt:
            wv.ad_af_dp.update({"AD": wv.smpl[wv.frmt.index("AD")]})

        wv.standardize_allele_frequency(log)

        wv.standardize_genotype(log)

        wv.standardize_depth(log)

        wv.standardize_allelic_depth(log)

    if vendsig_dict:
        wv.add_vendsig(vendsig_dict, log)

    wv.prune_var(log)

    updated_variant = wv.reassemble_variant()

    return updated_variant


def standardize_vcf(
    infile: str,
    outfile: str,
    out_path: str,
    case_id: str,
    log: Logger,
    vendsig_dict: dict = None,
    compression: bool = False,
) -> Optional[int]:
    check_vcf(infile, log)

    if compression and not outfile.endswith(".gz"):
        outfile = outfile + ".gz"

    headers = []
    with gzip.open(infile, "rt") if infile.endswith(".gz") else open(infile, "r") as f:
        headers = read_headers(f)

    line_count = len(headers)

    with gzip.open(infile, "rt") if infile.endswith(".gz") else open(infile, "r") as f:
        input_variants = read_variants(f)

        def output_variants_gen() -> Iterator[Optional[str]]:
            for variant in input_variants:
                formatted_variant = format_variant(variant, log, vendsig_dict)
                yield formatted_variant

        standardized_headers = add_to_headers(headers, case_id)
        output_variants = output_variants_gen()

        log.info(f"Writing standardized VCF to {out_path}/{outfile}")
        line_count = write_vcf(
            standardized_headers,
            output_variants,
            f"{out_path}/{outfile}",
            compression,
            line_count,
            log,
        )
        return line_count



import gzip
import xmltodict
from natsort import natsorted
from logging import Logger
import re
import os
import errno

from ingestion.vcf_standardization.standardize import standardize_vcf


def vcf_etl(in_vcf: str, out_vcf: str, base_xml_name: str, xml_file: str, log: Logger) -> int:
    if not os.path.exists(os.path.dirname(out_vcf)):
        try:
            os.makedirs(os.path.dirname(out_vcf))
        except OSError as exc:
            if exc.errno != errno.EEXIST:
                raise

    # Get xml short variant entries for scraping vendsig
    xml_short_vars = get_xml_short_vars(xml_file, log)
    outfile = out_vcf.split("/")[-1]
    out_path = out_vcf.replace(f"/{outfile}", "")

    line_count = standardize_vcf(
        infile=in_vcf,
        outfile=outfile,
        out_path=out_path,
        case_id=base_xml_name,
        log=log,
        vendsig_dict=xml_short_vars,
        compression=True,
    )

    return line_count


def map_vendsig(vendsig: str) -> str:
    if vendsig in ["known"]:
        return "Pathogenic"
    elif vendsig in ["likely"]:
        return "Likely pathogenic"
    elif vendsig in ["unknown"]:
        return "Uncertain significance"
    else:
        return "Unknown"


def get_xml_short_vars(xml_file: str, log):
    vendsig_dict = {"vendor": "foundation"}

    # xml_dict returns a dictionary if only one variant present, otherwise a list of dictionaries
    with open(xml_file) as fd:
        xml_dict = xmltodict.parse(fd.read())
    try:
        xml_short_vars = xml_dict["rr:ResultsReport"]["rr:ResultsPayload"]["variant-report"][
            "short-variants"
        ]["short-variant"]
    except TypeError:
        log.info("No short variants found in xml")
        return vendsig_dict

    xml_short_vars = [xml_short_vars] if isinstance(xml_short_vars, dict) else xml_short_vars
    for var in xml_short_vars:
        gene_id = var.get("@gene", "")
        depth = var.get("@depth", "")
        lookup_key = f"{gene_id}:{depth}"

        vendsig = var.get("@status", "")
        mapped_vendsig = map_vendsig(vendsig)

        vendsig_dict.update({lookup_key: mapped_vendsig})

    return vendsig_dict



import os
from logging import Logger
from schema import Schema, SchemaError, Optional
from ruamel.yaml import YAML


def check_manifest(manifest_file_path: str, case_id, log: Logger):
    manifest_schema = Schema(
        {
            # Required fields
            "name": str,  # Vendor name
            "testType": str,  # Vendor test name
            "indexedDate": str,  # Date of ingestion YYYY-MM-DD
            "reference": str,  # Reference genome
            "mrn": str,  # Patient MRN
            "patientInfo": {
                "lastName": str,
                "dob": str,  # YYYY-MM-DD
                "firstName": str,
                "gender": str,
            },
            # Optional fields
            Optional("receivedDate"): str,
            Optional("collDate"): str,
            Optional("reportDate"): str,
            Optional("reportFile"): str,
            Optional("medFacilName"): str,
            Optional("medFacilID"): str,
            Optional("orderingMDName"): str,
            Optional("orderingMDNPI"): str,
            Optional("indicationSystem"): "http://lifeomic.com/fhir/sequence-indication",
            Optional("indication"): str,
            Optional("indicationDisplay"): str,
            Optional("bodySite"): str,
            Optional("bodySiteDisplay"): str,
            Optional("bodySiteSystem"): "http://lifeomic.com/fhir/sequence-body-site",
            Optional("tmb"): str,
            Optional("tmbScore"): float,
            Optional("msi"): str,
            Optional("lossOfHeterozygosityScore"): int,
            Optional("lossOfHeterozygosityStatus"): str,
            Optional("ihcTests"): any,
            Optional("nonHumanContent"): any,
            Optional("plasmaTumorFraction"): str,
            Optional("cellPurity"): float,
            Optional("hrdStatus"): str,
        }
    )

    # Read in manifest yaml
    if os.path.exists(manifest_file_path):
        with open(manifest_file_path, "r") as file:
            manifest = YAML(typ="safe")
            manifest = manifest.load(file)
    else:
        raise FileNotFoundError(f"Manifest file not found: {manifest_file_path}")

    # Validate
    try:
        manifest_schema.validate(manifest)
    except SchemaError as e:
        log.error(e)
        raise e

    # Add duplicate fields from supplied ones to fit formatting
    manifest["patientInfo"]["identifiers"] = [
        {
            "codingCode": "MR",
            "codingSystem": "http://hl7.org/fhir/v2/0203",
            "value": manifest["mrn"],
        }
    ]

    manifest["patientLastName"] = manifest["patientInfo"]["lastName"]

    manifest["patientDOB"] = manifest["patientInfo"]["dob"]

    manifest["reportID"] = case_id  # Vendor report ID / Case ID

    log.info(f"Manifest file validated")
    return manifest



def try_move_af_value(self):
    # Relocate and transform AF from the FORMAT section to the INFO section for each variant
    zipped = dict(zip([i.upper() for i in self.frmt], self.smpl))
    af = float(zipped.get("AF", False))
    if not af:
        af = float(zipped.get("VF", False))
        if not af:
            return self

    self.info.update({"AF": "{:.4f}".format(af)})
    self.ad_af_dp["AF"] = "{:.4f}".format(af)
    return self


def try_calculate_af(self):
    # Recalculate AF values from AD, includes biallelic/multiallelic handling for future decomposition
    # i.e. chr8	127790148	.	CTTT	C,CT	7243.73	.	SOR=0.309	GT:AD:DP	1/2:5,189,62:256
    ad_split = self.ad_af_dp["AD"].split(",")

    afs = []
    # Start at the first REF allele
    for ad in ad_split[1:]:
        if ad == "0":
            af = 0.0
        else:
            af = float(int(ad) / sum([int(i) for i in ad_split]))
        afs.append("{:.4f}".format(af))

    self.info.update({"AF": "{:.4f}".format(af)})
    self.ad_af_dp["AF"] = "{:.4f}".format(af)

    return self



import os

from ingestion.vcf_standardization.standardize import standardize_vcf
from ingestion.generic.utils import check_manifest
from lifeomic_logging import scoped_logger


def process(
    manifest_file: str, vcf_file: str, source_file_id: str, out_path: str, case_id: str
) -> dict[str, str]:
    with scoped_logger(__name__) as log:

        # Read in supplied manifest
        manifest = check_manifest(manifest_file, case_id, log)

        # Process VCF
        base_vcf_file = os.path.basename(vcf_file)
        vcf_out = base_vcf_file.replace(".vcf", ".modified.vcf")
        vcf_final = base_vcf_file.replace(".vcf", ".modified.nrm.filtered.vcf")
        if not vcf_final.endswith(".gz"):
            vcf_final = vcf_final + ".gz"
        vcf_line_count = standardize_vcf(
            vcf_file, vcf_out, out_path, case_id, log, compression=True
        )

        # Add to manifest
        manifest["sourceFileId"] = source_file_id
        manifest["resources"] = [{"fileName": f".lifeomic/vcf-ingest/{case_id}/{base_vcf_file}"}]
        manifest["files"] = [
            {
                "fileName": f".lifeomic/vcf-ingest/{case_id}/{vcf_final}",
                "sequenceType": "germline",
                "type": "shortVariant",
            }
        ]

        case_metadata = {
            "test_type": manifest["testType"],
            "vcf_line_count": vcf_line_count,
            "case_id": manifest["reportID"],
            "germline_genome_reference": manifest["reference"],
        }

        return case_metadata, manifest



import pandas as pd
from logging import Logger
from typing import TypedDict

from ingestion.shared_util.coords_to_genes import coords_to_genes
from ingestion.nextgen.util.variant_table import extract_variant_table
from ingestion.nextgen.util.interpretation import map_interpretation


class StructuralVariant(TypedDict):
    sample_id: str
    gene1: str
    gene2: str
    effect: str
    position_one: tuple[str, str, str]
    position_two: tuple[str, str, str]
    interpretation: str
    sequence_type: str
    in_frame: str
    attributes: dict


def structural_variant_to_csv_row(structural_variant: StructuralVariant) -> str:
    csv_row = ""
    for value in structural_variant.values():
        if isinstance(value, tuple):
            csv_row += ",".join(value)
        else:
            csv_row += f"{value}"
        csv_row += ","
    return f"{csv_row[:-1]}\n"


def are_variants_duplicates(sv1: StructuralVariant, sv2: StructuralVariant) -> bool:
    return (sv1["position1"] == sv2["position1"] and sv1["position2"] == sv2["position2"]) or (
        sv1["position1"] == sv2["position2"] and sv1["position2"] == sv2["position1"]
    )


def process_structural(sv_in_file: str, xml_in_file, root_path: str, prefix: str, log: Logger):
    structural_variant_table = extract_variant_table(
        xml_in_file=xml_in_file, variant_type="structural", log=log
    )

    structural_variant_path_name = f"{root_path}/{prefix}.structural.csv"
    sample_id = prefix

    with open(sv_in_file, "r") as f:
        variants = [line for line in f.readlines() if not line.startswith("#")]

    if not variants:
        log.info(f"No structural variants found in {sv_in_file}")
        structural_status = False
        return None, structural_status

    structural_variants: list[StructuralVariant] = []
    structural_status = True
    for variant in variants:
        working_variant = variant.strip().split("\t")

        chromosome1 = f"chr{working_variant[0]}"
        start_position1 = working_variant[1]

        if "MantaDEL" in working_variant[2] or "MantaDUP" in working_variant[2]:
            end_position1 = working_variant[7].split(";")[0].split("=")[1]
            chromosome2 = chromosome1
            start_position2 = start_position1
            end_position2 = end_position1
            effect = "deletion" if "MantaDEL" in working_variant[2] else "duplication"

            # Get genes from coordinates using center point of start and end positions
            gene1 = coords_to_genes(
                "GRCh38", chromosome1, int((int(start_position1) + int(end_position1)) / 2), log
            )
            gene2 = "N/A"

        else:
            alt = working_variant[4].strip("][TCGA").split(":")

            end_position1 = start_position1
            chromosome2 = f"chr{alt[0]}"
            start_position2 = alt[1]
            end_position2 = alt[1]
            effect = "translocation"

            # Get genes from coordinates using center point of start and end positions
            gene1 = coords_to_genes(
                "GRCh38", chromosome1, int((int(start_position1) + int(end_position1)) / 2), log
            )
            gene2 = coords_to_genes(
                "GRCh38", chromosome2, int((int(start_position2) + int(end_position2)) / 2), log
            )

        # Scrape interpretation
        interpretation = "unknown"
        if not structural_variant_table.empty:
            for _, row in structural_variant_table.iterrows():
                ref_coords: set[str] = set(row["gene"].split(" ")[1].strip("()").split(";"))
                variant_coords = set(
                    [f"{chromosome1}:{start_position1}", f"{chromosome2}:{start_position2}"]
                )

                if ref_coords == variant_coords:
                    interpretation = map_interpretation(row["info"], log)

        # Hard-code
        sequence_type = "Somatic"
        in_frame = "Unknown"
        attributes = {}

        structural_variants.append(
            {
                "sample_id": sample_id,
                "gene1": gene1,
                "gene2": gene2,
                "effect": effect,
                "position1": (chromosome1, start_position1, end_position1),
                "position2": (chromosome2, start_position2, end_position2),
                "interpretation": interpretation,
                "sequence_type": sequence_type,
                "in_frame": in_frame,
                "attributes": attributes,
            }
        )

    # Dedupe structural variants based on chromosome and positions
    deduped_structural_variants = []
    for sv in structural_variants:
        maybe_matching_variant = next(
            (
                variant
                for variant in deduped_structural_variants
                if are_variants_duplicates(sv, variant)
            ),
            None,
        )
        if not maybe_matching_variant:
            deduped_structural_variants.append(sv)

    log.info(f"Saving file to {structural_variant_path_name}")
    with open(structural_variant_path_name, "w+") as f:
        f.write(
            "sample_id,gene1,gene2,effect,chromosome1,start_position1,end_position1,chromosome2,start_position2,end_position2,interpretation,sequence_type,in-frame,attributes\n"
        )
        for sv in deduped_structural_variants:
            f.write(structural_variant_to_csv_row(sv))

    return structural_variant_path_name, structural_status
